@@ -246,29 +246,42 @@ gnome_print_job_get_pages (GnomePrintJob *job)
  * @width: 
  * @height: 
  * 
- * Get imaging area size available to application (TRUE sucess,
- * FALSE error). It takes a config for argument, as you may not
- * want to create the job yet. Sizes are given in PS points
- * (GNOME_PRINT_PS_UNIT)
+ * Deprecated, use gnome_print_config_get_page_size
  * 
  * Return Value: TRUE on success, FALSE on error
  **/
 gboolean
 gnome_print_job_get_page_size_from_config (GnomePrintConfig *config, gdouble *width, gdouble *height)
 {
-	GnomePrintJob *job;
+	return gnome_print_config_get_page_size (config, width, height);
+}
+
+/**
+ * gnome_print_job_get_page_size:
+ * @job: 
+ * @width: 
+ * @height: 
+ * 
+ * Get the imaging area size that is available to the application
+ * Sizes are given in PS points (GNOME_PRINT_PS_UNIT)
+ * 
+ * Return Value: TRUE on success, FALSE on error
+ **/
+gboolean
+gnome_print_job_get_page_size (GnomePrintJob *job, gdouble *width, gdouble *height)
+{
 	JOBPrivate *pp;
 
-	g_return_val_if_fail (config != NULL, FALSE);
-
-	job = gnome_print_job_new (config);
+	g_return_val_if_fail (job != NULL, FALSE);
+	g_return_val_if_fail (GNOME_IS_PRINT_JOB (job), FALSE);
+	g_return_val_if_fail (width != NULL, FALSE);
+	g_return_val_if_fail (height != NULL, FALSE);
 
 	pp = job->priv;
 
 	job_update_layout_data (job);
 
 	if (pp->LY_LIST) {
-		/* Success */
 		if (width)
 			*width = pp->LW;
 		if (height)
